<?xml version="1.0" encoding="UTF-8"?>
<!--
/**
 * Phoebe pedagogic planner
 *
 * LICENSE: The GNU General Public License (GPL), version 2. See LICENSE.txt
 *
 * Copyright (c) 2006-2008 JISC/HEFCE/University of Oxford
 *
 * @author      David Balch <david.balch@conted.ox.ac.uk>
 * @license     http://opensource.org/licenses/gpl-license.php GPL2
 * @link        http://phoebe-project.conted.ox.ac.uk/
 */
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
    <!--
    Convert Phoebe design XML to HTML for presentation -->
    <xsl:template match="/data">
        <div id="designInfo">
            <h2><xsl:value-of select="designInfo/title"/></h2>
            <div><xsl:value-of select="designInfo/description"/></div>
            <p>
                <xsl:text>Created by: </xsl:text>
                <xsl:value-of select="designInfo/fullname"/>
                <xsl:if test="not(designInfo/email = '')">
                    <xsl:text> (</xsl:text>
                    <xsl:value-of select="designInfo/email"/>
                    <xsl:text>)</xsl:text>
                </xsl:if>
            </p>
        </div>
        <xsl:apply-templates select="props/children/item"/>
    </xsl:template>
    
    <xsl:template match="item">
        <xsl:variable name="DEPTH" select="count(ancestor::children)"/>
        <div class="property">
            <xsl:choose>
                <xsl:when test="field_type = '0'">
                    <!-- No input: just a heading -->
                    <xsl:element name="h{$DEPTH + 2}">
                        <xsl:value-of select="title"/>
                    </xsl:element>
                </xsl:when>

                <xsl:when test="field_type = '1'">
                    <!-- Small (single line) input -->
                    <xsl:element name="h{$DEPTH + 2}">
                        <xsl:attribute name="class">smallHeading</xsl:attribute>
                        <xsl:value-of select="title"/>
                    </xsl:element>
                        <xsl:text>: </xsl:text>
                        <xsl:if test="not(value = '')">
                            <xsl:for-each select="value">
                                <xsl:copy-of select="node()"/>
                            </xsl:for-each>
                        </xsl:if>
                </xsl:when>

                <xsl:when test="field_type = '2'">
                    <!-- large input -->
                    <xsl:element name="h{$DEPTH + 2}">
                        <xsl:value-of select="title"/>
                    </xsl:element>
                    <xsl:if test="not(value = '')">
                        <xsl:for-each select="value">
                            <div class="value">
                                <xsl:copy-of select="node()"/>
                            </div>
                        </xsl:for-each>
                    </xsl:if>
                </xsl:when>

                <xsl:when test="field_type = '3'">
                    <!-- Reference to another property -->
                    <xsl:element name="h{$DEPTH + 2}">
                        <xsl:value-of select="title"/>
                    </xsl:element>
                </xsl:when>

            </xsl:choose>
            <xsl:apply-templates select="children/item"/>
        </div>
    </xsl:template>
</xsl:stylesheet>
