<?xml version="1.0" encoding="UTF-8"?>
<!--
/**
 * Phoebe pedagogic planner
 *
 * LICENSE: The GNU General Public License (GPL), version 2. See LICENSE.txt
 *
 * Copyright (c) 2006-2008 JISC/HEFCE/University of Oxford
 *
 * @author      David Balch <david.balch@conted.ox.ac.uk>
 * @license     http://opensource.org/licenses/gpl-license.php GPL2
 * @link        http://phoebe-project.conted.ox.ac.uk/
 */
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
    <!--
    Convert Phoebe design XML to HTML for presentation -->
    <xsl:template match="/data">
        <div id="designInfo">
            <h2><xsl:value-of select="designInfo/title"/></h2>
            <div><xsl:value-of select="designInfo/description"/></div>
            <p>
                <xsl:text>Created by: </xsl:text>
                <xsl:value-of select="designInfo/fullname"/>
                <xsl:if test="not(designInfo/email = '')">
                    <xsl:text> (</xsl:text>
                    <xsl:value-of select="designInfo/email"/>
                    <xsl:text>)</xsl:text>
                </xsl:if>
            </p>
        </div>
        <xsl:apply-templates select="props/children/item"/>
    </xsl:template>

    <xsl:template match="item">
        <xsl:variable name="DEPTH" select="count(ancestor::children)"/>
        <xsl:choose>
            <xsl:when test="$DEPTH = '1'">
                <table>
                    <caption><xsl:value-of select="title"/></caption>
                    <tbody>
                    <xsl:if test="not(value = '')">
                        <tr><td colspan="2">
                            <xsl:for-each select="value">
                                <div class="value">
                                    <xsl:copy-of select="node()"/>
                                </div>
                            </xsl:for-each>
                        </td></tr>
                    </xsl:if>
                    <xsl:apply-templates select="children/item"/>
                    </tbody>
                </table>
            </xsl:when>

            <xsl:otherwise>
                <xsl:choose>
                    <xsl:when test="not(value = '') and not(children/item)">
                        <!-- Have a field value, and no children -->
                        <tr>
                            <th><xsl:value-of select="title"/></th>
                            <td>
                                <xsl:for-each select="value">
                                    <xsl:copy-of select="node()"/>
                                </xsl:for-each>
                            </td>
                        </tr>
                    </xsl:when>
                    <xsl:when test="not(value = '') and children/item">
                        <!-- Have a field value, and children -->
                        <tr>
                            <th rowspan="2"><xsl:value-of select="title"/></th>
                            <td>
                                <xsl:for-each select="value">
                                    <xsl:copy-of select="node()"/>
                                </xsl:for-each>
                            </td>
                        </tr>
                        <tr>
                            <td class="parentCell"><table><tbody><xsl:apply-templates select="children/item"/></tbody></table></td>
                        </tr>
                    </xsl:when>
                    <xsl:when test="value = '' and children/item">
                        <!-- Have no field value, but have children -->
                        <tr>
                            <th><xsl:value-of select="title"/></th>
                            <td class="parentCell"><table><tbody><xsl:apply-templates select="children/item"/></tbody></table></td>
                        </tr>
                    </xsl:when>
                    <xsl:when test="value = '' and not(children/item)">
                        <!-- Have no field value, and no children -->
                        <tr>
                            <th><xsl:value-of select="title"/></th>
                            <td></td>
                        </tr>
                    </xsl:when>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>